// Imperative and Object-Oriented Programming Methodology
// Uppsala University - 2018 Autumn
// Copyright © 2018 Henrik Schulze & Tobias Wrigstad
/// Här skall du skriva din kod:
/**
 *
 * @author henke-hp15bw056no
 * @param <T>
 */
public class List<T> implements Seq<T> {

  private int size = 0;
  Node first = null;
  Node last = null;

  private class Node {

    T elem;
    Node next;

    Node(T e, Node n) {
      elem = e;
      next = n;
    }
  }

  /**
   *
   * @param element
   */
  @Override
  public void append(T element) {
    if (size == 0) {
      last = first = new Node(element, null);
    } else {
      last.next = new Node(element, null);
      last = last.next;
    }
    ++size;
  }

  /**
   *
   * @param element
   */
  @Override
  public void prepend(T element) {
    if (size == 0) {
      last = first = new Node(element, null);
    } else {
      first = new Node(element, first);
    }
    ++size;
  }

  /**
   *
   * @param element
   */
  @Override
  public void remove(T element) {
    if (first != null) {
      if (element.equals(first.elem)) {
        first = first.next;
        if (first == null) {
          last = null;
        }
        --size;
      } else {
        Node prev = first;
        Node cursor = first.next;
        for (int i = 0; cursor != null && i < size; ++i) {
          if (element.equals(cursor.elem)) {
            prev.next = cursor.next;
            if (cursor.next == null) {
              last = prev;
            }
            --size;
            cursor = null;
          } else {
            prev = prev.next;
            cursor = cursor.next;
          }
        }
      }
    }
  }

  /**
   *
   * @param element
   * @return
   */
  @Override
  public boolean contains(T element) {
    boolean contains = false;
    Node cursor = first;
    for (int i = 0; cursor != null && i < size; ++i) {
      if (element.equals(cursor.elem)) {
        contains = true;
        cursor = null;
      } else {
        cursor = cursor.next;
      }
    }
    return contains;
  }

  /**
   *
   * @return
   */
  @Override
  public int size() {
    return size;
  }

  /**
   *
   * @param index
   * @return
   */
  @Override
  public T get(int index) {
    Node cursor = first;
    for (int i = 0; i < index; ++i) {
      cursor = cursor.next;
    }
    return cursor.elem;
  }

  /**
   *
   * @return
   */
  @Override
  public T first() {
    if (first != null) {
      return first.elem;
    }
    return null;
  }

  /**
   *
   * @return
   */
  @Override
  public T last() {
    if (last != null) {
      return last.elem;
    }
    return null;
  }

  /**
   * Removes the first element in the list and inserts it last. Example: list == [1, 2, 3]
   * list.rotate() list == [2, 3, 1]
   */
  @Override
  public void rotate() {
    last.next = first;
    first = first.next;
    last = last.next;
    last.next = null;
  }

  @Override
  public String toString() {
    String toString = "[ ";
    if (size != 0) {
      Node cursor = first;
      for (int i = 0; i < size; ++i) {
        toString += cursor.elem + " ";
        cursor = cursor.next;
      }
    }
    return toString + "], size = " + size;
  }
}
