// Imperative and Object-Oriented Programming Methodology
// Uppsala University - 2018 Autumn
// Copyright © 2018 Henrik Schulze & Tobias Wrigstad
/// Här skall du skriva din kod:
public class List<T> implements Seq<T> {

  private int size = 0;
  private final T[] theList = (T[]) new Object[31]; // 31 is arbitrary

  /// Inserts an element at the back of a list
  /// Example:
  /// list == [1, 2]
  /// list.append(3)
  /// list == [1, 2, 3]
  @Override
  public void append(T element) {
    theList[size] = element;
    ++size;
  }

  /// Inserts an element at the front of a list
  /// Example:
  /// list == [1, 2]
  /// list.prepend(3)
  /// list == [3, 1, 2]
  @Override
  public void prepend(T element) {
    for (int i = size; i > 0; --i) {
      theList[i] = theList[i - 1];
    }
    theList[0] = element;
    ++size;
  }

  /// Removes the first occurrence of an element from the list
  /// Example 1:
  /// list == [1, 2, 1]
  /// list.remove(1)
  /// list == [2, 1]
  ///
  /// Example 2:
  /// list == [1, 2, 1]
  /// list.remove(3)
  /// list == [1, 2, 1]
  @Override
  public void remove(T element) {
    boolean contains = false;
    for (int i = 0; !contains && i < size; ++i) {
      contains = (theList[i].equals(element));
      if (contains) {
        for (int j = i; j < size; ++j) {
          theList[j] = theList[j + 1];
        }
        --size;
      }
    }
  }

  /// Returns true if a given element is in the list
  /// Example:
  /// list == [1, 2, 3]
  /// list.contains(2) ==> true
  /// list.contains(4) ==> false
  /// NOTE: element comparison is structural, i.e. not based on identity comparison,
  /// Example:
  /// list.append(new String("Deeeo!"))
  /// list.contains(new String("Deeeo!")) ==> true
  @Override
  public boolean contains(T element) {
    boolean contains = false;
    for (int i = 0; !contains && i < size; ++i) {
      contains = (theList[i].equals(element));
    }
    return contains;
  }

  /// Returns the size of the list
  /// Example:
  /// list == [1, 2, 3]
  /// list.size() ==> 3
  @Override
  public int size() {
    return size;
  }

  /// Returns the ith element in the list
  /// Example:
  /// list == [1, 2, 3]
  /// list.get(0) ==> 1
  /// list.get(1) ==> 2
  /// list.get(2) ==> 3
  /// list.get(n) ==> throws IllegalArgumentException when n not in [0..list.size()-1]
  @Override
  public T get(int index) {
    if (!(0 <= index && index < size)) {
      throw new IllegalArgumentException();
    }
    return theList[index];
  }

  /// Returns the first element in the list
  /// Example:
  /// list == [1, 2, 3]
  /// list.first() ==> 1
  @Override
  public T first() {
    return theList[0];
  }

  /// Returns the last element in the list
  /// Example:
  /// list == [1, 2, 3]
  /// list.last() ==> 3
  @Override
  public T last() {
    return theList[size - 1];
  }

  /// Removes the first element in the list and inserts it last
  /// Example:
  /// list == [1, 2, 3]
  /// list.rotate()
  /// list == [2, 3, 1]
  @Override
  public void rotate() {
    T first = theList[0];
    remove(first);
    append(first);
//    System.out.println(" this = " + this);
  }

  @Override
  public String toString() {
    String toString = " theList = [";
    if (size > 0) {
      for (int i = 0; i < size - 1;) {
        toString += theList[i] + ",";
        ++i;
      }
      toString += theList[size - 1];
    }
    return toString + "] & size = " + size;
  }
}
